@@ -48,12 +48,11 @@ static ulonglong system_variable_hash_version= 0;
   Return variable name and length for hashing of variables.
 */
 
-static const uchar *get_sys_var_length(const void *var_, size_t *length,
-                                       my_bool)
+static uchar *get_sys_var_length(const sys_var *var, size_t *length,
+                                 my_bool first)
 {
-  auto var= static_cast<const sys_var *>(var_);
   *length= var->name.length;
-  return reinterpret_cast<const uchar *>(var->name.str);
+  return (uchar*) var->name.str;
 }
 
 sys_var_chain all_sys_vars = { NULL, NULL };
@@ -65,9 +64,8 @@ int sys_var_init()
   /* Must be already initialized. */
   DBUG_ASSERT(system_charset_info != NULL);
 
-  if (my_hash_init(PSI_INSTRUMENT_ME, &system_variable_hash,
-                   system_charset_info, 700, 0, 0, get_sys_var_length, 0,
-                   HASH_UNIQUE))
+  if (my_hash_init(PSI_INSTRUMENT_ME, &system_variable_hash, system_charset_info, 700, 0,
+                   0, (my_hash_get_key) get_sys_var_length, 0, HASH_UNIQUE))
     goto error;
 
   if (mysql_add_sys_var_chain(all_sys_vars.first))
@@ -87,7 +85,7 @@ uint sys_var_elements()
 
 int sys_var_add_options(DYNAMIC_ARRAY *long_options, int parse_flags)
 {
-  uint saved_elements= long_options->elements;
+  size_t saved_elements= long_options->elements;
 
   DBUG_ENTER("sys_var_add_options");
 
@@ -216,8 +214,9 @@ bool sys_var::update(THD *thd, set_var *var)
 
     /*
       Make sure we don't session-track variables that are not actually
-      part of the session. tx_isolation and and tx_read_only for example
-      exist as GLOBAL, SESSION, and one-shot ("for next transaction only").
+      part of the session. transaction_isolation and transaction_read_only for
+      example exist as GLOBAL, SESSION, and one-shot ("for next transaction
+      only").
     */
     if ((var->type == OPT_SESSION) && (!ret))
     {
@@ -312,7 +311,7 @@ do {                                                \
     case SHOW_HA_ROWS:  do_num_val (ha_rows,CMD);
 
 #define case_for_double(CMD)                        \
-    case SHOW_DOUBLE:   do_num_val (double,CMD)
+    case SHOW_DOUBLE:   do_num_val (double,CMD);
 
 #define case_get_string_as_lex_string               \
     case SHOW_CHAR:                                 \
@@ -425,20 +424,7 @@ void sys_var::do_deprecated_warning(THD *thd)
   {
     char buf1[NAME_CHAR_LEN + 3];
     strxnmov(buf1, sizeof(buf1)-1, "@@", name.str, 0);
-
-    /* 
-       if deprecation_substitute is an empty string,
-       there is no replacement for the syntax
-    */
-    uint errmsg= deprecation_substitute[0] == '\0'
-      ? ER_WARN_DEPRECATED_SYNTAX_NO_REPLACEMENT
-      : ER_WARN_DEPRECATED_SYNTAX;
-    if (thd)
-      push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
-                          ER_WARN_DEPRECATED_SYNTAX, ER_THD(thd, errmsg),
-                          buf1, deprecation_substitute);
-    else
-      sql_print_warning(ER_DEFAULT(errmsg), buf1, deprecation_substitute);
+    warn_deprecated<999999>(thd, buf1, deprecation_substitute);
   }
 }
 
@@ -535,7 +521,6 @@ static my_old_conv old_conv[]=
 CHARSET_INFO *get_old_charset_by_name(const char *name)
 {
   my_old_conv *conv;
-
   for (conv= old_conv; conv->old_name; conv++)
   {
     if (!my_strcasecmp(&my_charset_latin1, name, conv->old_name))
@@ -616,10 +601,8 @@ int mysql_del_sys_var_chain(sys_var *first)
 }
 
 
-static int show_cmp(const void *a_, const void *b_)
+static int show_cmp(SHOW_VAR *a, SHOW_VAR *b)
 {
-  const SHOW_VAR *a= static_cast<const SHOW_VAR *>(a_);
-  const SHOW_VAR *b= static_cast<const SHOW_VAR *>(b_);
   return strcmp(a->name, b->name);
 }
 
@@ -795,6 +778,7 @@ int set_var::check(THD *thd)
   if (!value)
     return 0;
 
+  thd->where= THD_WHERE::SET_LIST;
   if (value->fix_fields_if_needed_for_scalar(thd, &value))
     return -1;
   if (var->check_update_type(value))
@@ -830,15 +814,20 @@ int set_var::check(THD *thd)
 */
 int set_var::light_check(THD *thd)
 {
+  if (var->is_readonly())
+  {
+    my_error(ER_INCORRECT_GLOBAL_LOCAL_VAR, MYF(0), var->name.str, "read only");
+    return -1;
+  }
   if (var->check_type(type))
   {
     int err= type == OPT_GLOBAL ? ER_LOCAL_VARIABLE : ER_GLOBAL_VARIABLE;
     my_error(err, MYF(0), var->name.str);
     return -1;
   }
-  if (type == OPT_GLOBAL &&
-      check_global_access(thd, PRIV_SET_GLOBAL_SYSTEM_VARIABLE))
-    return 1;
+
+  if (type == OPT_GLOBAL && var->on_check_access_global(thd))
+      return 1;
 
   if (value && value->fix_fields_if_needed_for_scalar(thd, &value))
     return -1;
@@ -896,6 +885,7 @@ int set_var_user::check(THD *thd)
     Item_func_set_user_var can't substitute something else on its place =>
     0 can be passed as last argument (reference on item)
   */
+  thd->where= THD_WHERE::SET_LIST;
   return (user_var_item->fix_fields(thd, (Item**) 0) ||
           user_var_item->check(0)) ? -1 : 0;
 }
@@ -1038,7 +1028,7 @@ int set_var_collation_client::check(THD *thd)
   if (!is_supported_parser_charset(character_set_client))
   {
     my_error(ER_WRONG_VALUE_FOR_VAR, MYF(0), "character_set_client",
-             character_set_client->csname);
+             character_set_client->cs_name.str);
     return 1;
   }
   return 0;
@@ -1093,7 +1083,7 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
   StringBuffer<STRING_BUFFER_USUAL_SIZE> strbuf(scs);
   const char *wild= thd->lex->wild ? thd->lex->wild->ptr() : 0;
   Field **fields=tables->table->field;
-  bool has_file_acl= !check_access(thd, FILE_ACL, any_db, NULL, NULL, 0, 1);
+  bool has_file_acl= !check_access(thd, FILE_ACL, any_db.str, NULL,NULL,0,1);
 
   DBUG_ASSERT(tables->table->in_use == thd);
 
@@ -1112,7 +1102,7 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
     fields[0]->store(name_buffer, strlen(name_buffer), scs);
 
     if ((wild && wild_case_compare(system_charset_info, name_buffer, wild))
-        || (cond && !cond->val_int()))
+        || (cond && !cond->val_bool()))
       continue;
 
     mysql_mutex_lock(&LOCK_global_system_variables);
@@ -1223,12 +1213,14 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
     {
       uint i;
       strbuf.length(0);
-      for (i=0; i + 1 < tl->count; i++)
+      for (i=0; i < tl->count; i++)
       {
-        strbuf.append(tl->type_names[i]);
+        const char *name= tl->type_names[i];
+        strbuf.append(name, strlen(name));
         strbuf.append(',');
       }
-      strbuf.append(tl->type_names[i]);
+      if (!strbuf.is_empty())
+        strbuf.chop();
       fields[11]->set_notnull();
       fields[11]->store(strbuf.ptr(), strbuf.length(), scs);
     }
@@ -1548,4 +1540,3 @@ ulonglong get_system_variable_hash_version(void)
 {
   return system_variable_hash_version;
 }
-
