@@ -108,67 +108,63 @@ extern volatile ulonglong spider_mon_table_cache_version_req;
       MYSQL_SYSVAR_NAME(param_name).def_val;                            \
   }
 
-static int spider_direct_update(THD *thd, SHOW_VAR *var, void *,
-                                system_status_var *, enum_var_type)
+extern handlerton *spider_hton_ptr;
+static void spider_trx_status_var(THD *thd, SHOW_VAR *var, void *buff,
+                                 ulonglong SPIDER_TRX::*counter)
 {
-  int error_num = 0;
-  SPIDER_TRX *trx;
   DBUG_ENTER("spider_direct_update");
   var->type = SHOW_LONGLONG;
-  if ((trx = spider_get_trx(thd, TRUE, &error_num)))
-    var->value = (char *) &trx->direct_update_count;
-  DBUG_RETURN(error_num);
+  var->value= buff;
+  if (thd != current_thd)
+    mysql_mutex_lock(&thd->LOCK_thd_data);
+  SPIDER_TRX *trx = (SPIDER_TRX*)thd_get_ha_data(thd, spider_hton_ptr);
+  *(ulonglong*)buff= trx ? trx->*counter : 0;
+  if (thd != current_thd)
+    mysql_mutex_unlock(&thd->LOCK_thd_data);
+  DBUG_VOID_RETURN;
 }
 
-static int spider_direct_delete(THD *thd, SHOW_VAR *var, void *,
+
+static int spider_direct_update(THD *thd, SHOW_VAR *var, void *buff,
                                 system_status_var *, enum_var_type)
 {
-  int error_num = 0;
-  SPIDER_TRX *trx;
-  DBUG_ENTER("spider_direct_delete");
-  var->type = SHOW_LONGLONG;
-  if ((trx = spider_get_trx(thd, TRUE, &error_num)))
-    var->value = (char *) &trx->direct_delete_count;
-  DBUG_RETURN(error_num);
+  DBUG_ENTER("spider_direct_update");
+  spider_trx_status_var(thd, var, buff, &SPIDER_TRX::direct_update_count);
+  DBUG_RETURN(0);
 }
 
-static int spider_direct_order_limit(THD *thd, SHOW_VAR *var, void *,
+static int spider_direct_delete(THD *thd, SHOW_VAR *var, void *buff,
+                                system_status_var *, enum_var_type)
+{
+  DBUG_ENTER("spider_direct_delete");
+  spider_trx_status_var(thd, var, buff, &SPIDER_TRX::direct_delete_count);
+  DBUG_RETURN(0);
+}
+
+static int spider_direct_order_limit(THD *thd, SHOW_VAR *var, void *buff,
                                      system_status_var *, enum_var_type)
 {
-  int error_num = 0;
-  SPIDER_TRX *trx;
   DBUG_ENTER("spider_direct_order_limit");
-  var->type = SHOW_LONGLONG;
-  if ((trx = spider_get_trx(thd, TRUE, &error_num)))
-    var->value = (char *) &trx->direct_order_limit_count;
-  DBUG_RETURN(error_num);
+  spider_trx_status_var(thd, var, buff, &SPIDER_TRX::direct_order_limit_count);
+  DBUG_RETURN(0);
 }
 
-static int spider_direct_aggregate(THD *thd, SHOW_VAR *var, void *,
+static int spider_direct_aggregate(THD *thd, SHOW_VAR *var, void *buff,
                                    system_status_var *, enum_var_type)
 {
-  int error_num = 0;
-  SPIDER_TRX *trx;
   DBUG_ENTER("spider_direct_aggregate");
-  var->type = SHOW_LONGLONG;
-  if ((trx = spider_get_trx(thd, TRUE, &error_num)))
-    var->value = (char *) &trx->direct_aggregate_count;
-  DBUG_RETURN(error_num);
+  spider_trx_status_var(thd, var, buff, &SPIDER_TRX::direct_aggregate_count);
+  DBUG_RETURN(0);
 }
 
-static int spider_parallel_search(THD *thd, SHOW_VAR *var, void *,
+static int spider_parallel_search(THD *thd, SHOW_VAR *var, void *buff,
                                   system_status_var *, enum_var_type)
 {
-  int error_num = 0;
-  SPIDER_TRX *trx;
   DBUG_ENTER("spider_parallel_search");
-  var->type = SHOW_LONGLONG;
-  if ((trx = spider_get_trx(thd, TRUE, &error_num)))
-    var->value = (char *) &trx->parallel_search_count;
-  DBUG_RETURN(error_num);
+  spider_trx_status_var(thd, var, buff, &SPIDER_TRX::parallel_search_count);
+  DBUG_RETURN(0);
 }
 
-
 struct st_mysql_show_var spider_status_variables[] =
 {
   {"Spider_mon_table_cache_version",
@@ -215,6 +211,34 @@ static MYSQL_SYSVAR_BOOL(
   TRUE
 );
 
+static void spider_var_deprecated_int(THD *thd, st_mysql_sys_var *,
+                                      void *var_ptr, const void *save)
+{
+  int val= *static_cast<const int *>(save);
+  *static_cast<int *>(var_ptr)= val;
+  if (val == -1)
+  {
+    push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
+                        HA_ERR_UNSUPPORTED,
+                        "The option value -1 (fallback to default) is "
+                        "deprecated and will be removed in a future release");
+  }
+}
+
+static void spider_var_deprecated_longlong(THD *thd, st_mysql_sys_var *,
+                                           void *var_ptr, const void *save)
+{
+  longlong val= *static_cast<const longlong *>(save);
+  *static_cast<longlong *>(var_ptr)= val;
+  if (val == -1)
+  {
+    push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
+                        HA_ERR_UNSUPPORTED,
+                        "The option value -1 (fallback to default) is "
+                        "deprecated and will be removed in a future release");
+  }
+}
+
 SPIDER_SYSVAR_VALUE_FUNC(my_bool, support_xa)
 
 static my_bool spider_connect_mutex;
@@ -280,7 +304,7 @@ static MYSQL_SYSVAR_INT(
   PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
   "Use table charset for remote access",
   NULL,
-  NULL,
+  spider_var_deprecated_int,
   1,
   -1,
   1,
@@ -416,7 +440,7 @@ SPIDER_THDVAR_VALUE_FUNC(uint, force_commit)
  */
 static MYSQL_THDVAR_UINT(
   xa_register_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Mode of XA transaction register into system table", /* comment */
   NULL, /* check */
   NULL, /* update */
@@ -434,10 +458,10 @@ SPIDER_THDVAR_VALUE_FUNC(uint, xa_register_mode)
  */
 static MYSQL_THDVAR_LONGLONG(
   internal_offset, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Internal offset", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_longlong, /* update */
   0, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -452,10 +476,10 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(longlong, internal_offset)
  */
 static MYSQL_THDVAR_LONGLONG(
   internal_limit, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Internal limit", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_longlong, /* update */
   9223372036854775807LL, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -473,7 +497,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of rows at a select", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_longlong, /* update */
   9223372036854775807LL, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -492,7 +516,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Use offset and limit parameter in SQL for split_read parameter.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   2, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -510,7 +534,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The limit value for semi_split_read", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_longlong, /* update */
   9223372036854775807LL, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -526,10 +550,10 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(longlong, semi_split_read_limit)
  */
 static MYSQL_THDVAR_INT(
   init_sql_alloc_size, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Initial sql string alloc size", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1024, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -548,7 +572,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Reset sql string alloc after execute", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   1, /* max */
@@ -557,7 +581,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, reset_sql_alloc)
 
-
 /*
  -1 :fallback to default
   0 :off
@@ -568,7 +591,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Sprit read mode for multi range", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   100, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -586,7 +609,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Max columns for order by", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   32767, /* def */
   -1, /* min */
   32767, /* max */
@@ -657,7 +680,7 @@ static int spider_param_semi_table_lock_check(
  */
 static MYSQL_THDVAR_INT(
   semi_table_lock, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Table lock during execute a sql", /* comment */
   &spider_param_semi_table_lock_check, /* check */
   NULL, /* update */
@@ -711,10 +734,10 @@ static int spider_param_semi_table_lock_connection_check(
  */
 static MYSQL_THDVAR_INT(
   semi_table_lock_connection, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Use different connection if semi_table_lock is enabled", /* comment */
   &spider_param_semi_table_lock_connection_check, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   1, /* max */
@@ -751,7 +774,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Lock for select with update", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   2, /* max */
@@ -818,7 +841,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Bulk insert size", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   16000, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -840,7 +863,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The mode of bulk updating and deleting", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   2, /* max */
@@ -858,7 +881,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Bulk update size", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   16000, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -873,10 +896,10 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, bulk_update_size)
  */
 static MYSQL_THDVAR_INT(
   buffer_size, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Buffer size", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   16000, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -895,7 +918,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Execute optimize to remote server", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   1, /* max */
@@ -914,7 +937,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Execute optimize to remote server with local", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   1, /* max */
@@ -1026,7 +1049,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Wait timeout of connecting to remote server", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   6, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -1044,7 +1067,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Wait timeout of receiving data from remote server", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   600, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -1062,7 +1085,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Wait timeout of sending data to remote server", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   600, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -1084,7 +1107,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The retrieval result from a remote server is acquired by acquisition one by one", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   3, /* def */
   -1, /* min */
   3, /* max */
@@ -1102,7 +1125,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of records in a page when acquisition one by one", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1024, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -1120,7 +1143,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The limitation of memory size in a page when acquisition one by one", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   10485760, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -1139,7 +1162,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Use low memory mode when SQL(SELECT) internally issued to a remote server is executed and get a result list", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   1, /* max */
@@ -1159,7 +1182,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The mode of using columns at select clause", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   1, /* max */
@@ -1168,7 +1191,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, select_column_mode)
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 /*
  -1 :fallback to default
   0 :background search is disabled
@@ -1181,7 +1203,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of background search", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   3, /* max */
@@ -1200,7 +1222,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of first read records when background search is used", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   2, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -1219,7 +1241,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of second read records when background search is used", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   100, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -1227,7 +1249,6 @@ static MYSQL_THDVAR_LONGLONG(
 );
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(longlong, bgs_second_read)
-#endif
 
 /*
  -1 :fallback to default
@@ -1239,7 +1260,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of first read records", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -1258,7 +1279,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of second read records", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -1277,7 +1298,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Interval of cardinality confirmation.(second)", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   51, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -1295,10 +1316,10 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(double, crd_interval)
  */
 static MYSQL_THDVAR_INT(
   crd_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Mode of cardinality confirmation.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   3, /* max */
@@ -1307,7 +1328,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, crd_mode)
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
 /*
  -1 :fallback to default
   0 :No synchronization.
@@ -1320,7 +1340,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Cardinality synchronization in partitioned table.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   2, /* max */
@@ -1328,7 +1348,6 @@ static MYSQL_THDVAR_INT(
 );
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, crd_sync)
-#endif
 
 /*
  -1 :fallback to default
@@ -1338,10 +1357,10 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, crd_sync)
  */
 static MYSQL_THDVAR_INT(
   crd_type, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Type of cardinality calculation.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   2, /* def */
   -1, /* min */
   2, /* max */
@@ -1356,10 +1375,10 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, crd_type)
  */
 static MYSQL_THDVAR_INT(
   crd_weight, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Weight coefficient to calculate effectiveness of index from cardinality of column.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   2, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -1368,7 +1387,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(double, crd_weight)
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 /*
  -1 :fallback to default
   0 :Background confirmation is disabled
@@ -1380,7 +1398,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of cardinality confirmation at background.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   2, /* def */
   -1, /* min */
   2, /* max */
@@ -1388,7 +1406,6 @@ static MYSQL_THDVAR_INT(
 );
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, crd_bg_mode)
-#endif
 
 /*
  -1 :fallback to default
@@ -1400,7 +1417,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Interval of table state confirmation.(second)", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   10, /* def */
   -1, /* min */
   2147483647, /* max */
@@ -1417,10 +1434,10 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(double, sts_interval)
  */
 static MYSQL_THDVAR_INT(
   sts_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Mode of table state confirmation.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   2, /* max */
@@ -1429,11 +1446,10 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, sts_mode)
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
 /*
  -1 :fallback to default
   0 :No synchronization.
-  1 :Table state is synchronized when opening a table.
+  1 :Table stat is synchronized when opening a table.
      Then no synchronization.
   2 :Synchronization.
  */
@@ -1442,7 +1458,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Table state synchronization in partitioned table.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   2, /* max */
@@ -1450,9 +1466,7 @@ static MYSQL_THDVAR_INT(
 );
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, sts_sync)
-#endif
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 /*
  -1 :fallback to default
   0 :Background confirmation is disabled
@@ -1464,7 +1478,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of table state confirmation at background.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   2, /* def */
   -1, /* min */
   2, /* max */
@@ -1472,7 +1486,6 @@ static MYSQL_THDVAR_INT(
 );
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, sts_bg_mode)
-#endif
 
 /*
   0 :always ping
@@ -1492,7 +1505,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_VALUE_FUNC(double, ping_interval_at_trx_start)
 
-
 /*
  -1 :fallback to default
   0 :normal mode
@@ -1504,7 +1516,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of auto increment.", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   3, /* max */
@@ -1552,7 +1564,7 @@ SPIDER_THDVAR_VALUE_FUNC(bool, local_lock_table)
 static MYSQL_THDVAR_INT(
   use_pushdown_udf, /* name */
   PLUGIN_VAR_RQCMDARG, /* opt */
-  "Remote server transmission existence when UDF is used at condition and \"engine_condition_pushdown=1\"", /* comment */
+  "Remote server transmission existence when UDF is used at condition", /* comment */
   NULL, /* check */
   NULL, /* update */
   0, /* def */
@@ -1573,7 +1585,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Execute \"REPLACE\" and \"INSERT IGNORE\" on remote server and avoid duplicate check on local server", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   1, /* max */
@@ -1582,81 +1594,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, direct_dup_insert)
 
-static uint spider_udf_table_lock_mutex_count;
-/*
-  1-: mutex count
- */
-static MYSQL_SYSVAR_UINT(
-  udf_table_lock_mutex_count,
-  spider_udf_table_lock_mutex_count,
-  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
-  "Mutex count of table lock for Spider UDFs",
-  NULL,
-  NULL,
-  20,
-  1,
-  4294967295U,
-  0
-);
-
-SPIDER_SYSVAR_VALUE_FUNC(uint, udf_table_lock_mutex_count)
-
-static uint spider_udf_table_mon_mutex_count;
-/*
-  1-: mutex count
- */
-static MYSQL_SYSVAR_UINT(
-  udf_table_mon_mutex_count,
-  spider_udf_table_mon_mutex_count,
-  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
-  "Mutex count of table mon for Spider UDFs",
-  NULL,
-  NULL,
-  20,
-  1,
-  4294967295U,
-  0
-);
-
-SPIDER_SYSVAR_VALUE_FUNC(uint, udf_table_mon_mutex_count)
-
-/*
-  1-:number of rows
- */
-static MYSQL_THDVAR_LONGLONG(
-  udf_ds_bulk_insert_rows, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Number of rows for bulk inserting", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  3000, /* def */
-  -1, /* min */
-  9223372036854775807LL, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(longlong, udf_ds_bulk_insert_rows)
-
-/*
- -1 :fallback to default
-  0 :drop records
-  1 :insert last table
-  2 :insert first table and loop again
- */
-static MYSQL_THDVAR_INT(
-  udf_ds_table_loop_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Table loop mode if the number of tables in table list are less than the number of result sets", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  -1, /* min */
-  2, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, udf_ds_table_loop_mode)
-
 static char *spider_remote_access_charset;
 /*
  */
@@ -1803,7 +1740,7 @@ static MYSQL_THDVAR_INT(
   "Connect retry count", /* comment */
   NULL, /* check */
   NULL, /* update */
-  2, /* def */
+  1000, /* def */
   0, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1848,7 +1785,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of BKA for Spider", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   2, /* max */
@@ -1857,66 +1794,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, bka_mode)
 
-static int spider_udf_ct_bulk_insert_interval;
-/*
- -1         : Fallback to default.
-  0 or more : Milliseconds.
- */
-static MYSQL_SYSVAR_INT(
-  udf_ct_bulk_insert_interval,
-  spider_udf_ct_bulk_insert_interval,
-  PLUGIN_VAR_RQCMDARG,
-  "The interval time between bulk insert and next bulk insert at coping",
-  NULL,
-  NULL,
-  10,
-  -1,
-  2147483647,
-  0
-);
-
-SPIDER_SYSVAR_OVERRIDE_VALUE_FUNC(int, udf_ct_bulk_insert_interval)
-
-static longlong spider_udf_ct_bulk_insert_rows;
-/*
- -1,0       : Fallback to default.
-  1 or more : Number of rows.
- */
-static MYSQL_SYSVAR_LONGLONG(
-  udf_ct_bulk_insert_rows,
-  spider_udf_ct_bulk_insert_rows,
-  PLUGIN_VAR_RQCMDARG,
-  "The number of rows inserted with bulk insert of one time at coping",
-  NULL,
-  NULL,
-  100,
-  -1,
-  9223372036854775807LL,
-  0
-);
-
-SPIDER_SYSVAR_OVERRIDE_VALUE_FUNC(longlong, udf_ct_bulk_insert_rows)
-
-
-/*
- -1 :fallback to default
-  0 :not use
-  1 :use handler
- */
-static MYSQL_THDVAR_INT(
-  use_handler, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Use handler for reading", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  -1, /* min */
-  3, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, use_handler)
-
 /*
  -1 :fallback to default
   0 :return error if error
@@ -1927,7 +1804,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Read error mode if error", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   1, /* max */
@@ -1946,7 +1823,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Write error mode if error", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   1, /* max */
@@ -1965,7 +1842,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Skip generating internal default condition", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   1, /* max */
@@ -1986,7 +1863,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Skip parallel search by specific conditions", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   3, /* max */
@@ -2005,7 +1882,7 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Send 'ORDER BY' and 'LIMIT' to remote server directly", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_longlong, /* update */
   9223372036854775807LL, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
@@ -2024,7 +1901,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Read only", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   1, /* max */
@@ -2033,26 +1910,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, read_only_mode)
 
-
-/*
- -1 :fallback to default
-  0 :can not use
-  1 :can use
- */
-static MYSQL_THDVAR_INT(
-  udf_ds_use_real_table, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Use real table for temporary table list", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  -1, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, udf_ds_use_real_table)
-
 static my_bool spider_general_log;
 static MYSQL_SYSVAR_BOOL(
   general_log,
@@ -2158,17 +2015,6 @@ static MYSQL_SYSVAR_UINT(
 
 SPIDER_SYSVAR_VALUE_FUNC(uint, log_result_error_with_sql)
 
-static char *spider_version = (char *) SPIDER_DETAIL_VERSION;
-static MYSQL_SYSVAR_STR(
-  version,
-  spider_version,
-  PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_READONLY,
-  "The version of Spider",
-  NULL,
-  NULL,
-  SPIDER_DETAIL_VERSION
-);
-
 /*
   0: server_id + thread_id
   1: server_id + thread_id + query_id
@@ -2198,7 +2044,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Read casually if it is possible", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   63, /* max */
@@ -2230,7 +2076,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The type of delete_all_rows", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   1, /* max */
@@ -2246,10 +2092,10 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, delete_all_rows_type)
  */
 static MYSQL_THDVAR_INT(
   bka_table_name_type, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "The type of temporary table name for bka", /* comment */
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   0, /* def */
   -1, /* min */
   1, /* max */
@@ -2286,10 +2132,10 @@ static int spider_store_last_sts;
 static MYSQL_SYSVAR_INT(
   store_last_sts,
   spider_store_last_sts,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "Store last sts result into system table",
   NULL,
-  NULL,
+  spider_var_deprecated_int,
   1,
   -1,
   1,
@@ -2307,10 +2153,10 @@ static int spider_store_last_crd;
 static MYSQL_SYSVAR_INT(
   store_last_crd,
   spider_store_last_crd,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "Store last crd result into system table",
   NULL,
-  NULL,
+  spider_var_deprecated_int,
   1,
   -1,
   1,
@@ -2328,10 +2174,10 @@ static int spider_load_sts_at_startup;
 static MYSQL_SYSVAR_INT(
   load_sts_at_startup,
   spider_load_sts_at_startup,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "Load sts from system table at startup",
   NULL,
-  NULL,
+  spider_var_deprecated_int,
   1,
   -1,
   1,
@@ -2349,10 +2195,10 @@ static int spider_load_crd_at_startup;
 static MYSQL_SYSVAR_INT(
   load_crd_at_startup,
   spider_load_crd_at_startup,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "Load crd from system table at startup",
   NULL,
-  NULL,
+  spider_var_deprecated_int,
   1,
   -1,
   1,
@@ -2361,7 +2207,6 @@ static MYSQL_SYSVAR_INT(
 
 SPIDER_SYSVAR_OVERRIDE_VALUE_FUNC(int, load_crd_at_startup)
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 static uint spider_table_sts_thread_count;
 /*
   1-: thread count
@@ -2399,7 +2244,6 @@ static MYSQL_SYSVAR_UINT(
 );
 
 SPIDER_SYSVAR_VALUE_FUNC(uint, table_crd_thread_count)
-#endif
 
 static int spider_slave_trx_isolation;
 /*
@@ -2499,7 +2343,7 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Use columns in select clause strictly for group by clause",
   NULL, /* check */
-  NULL, /* update */
+  spider_var_deprecated_int, /* update */
   1, /* def */
   -1, /* min */
   1, /* max */
@@ -2524,6 +2368,41 @@ static MYSQL_THDVAR_BOOL(
 
 SPIDER_THDVAR_VALUE_FUNC(bool, direct_aggregate)
 
+static MYSQL_THDVAR_BOOL(
+  disable_group_by_handler, /* name */
+  PLUGIN_VAR_OPCMDARG, /* opt */
+  "Disables the group by handler", /* comment */
+  NULL, /* check */
+  NULL, /* update */
+  FALSE /* def */
+);
+
+SPIDER_THDVAR_VALUE_FUNC(bool, disable_group_by_handler)
+
+static MYSQL_THDVAR_BOOL(
+  suppress_comment_ignored_warning,
+  PLUGIN_VAR_RQCMDARG,
+  "Whether to suppress warnings that table COMMENT or CONNECTION strings "
+  "are ignored due to specified table options",
+  NULL,
+  NULL,
+  FALSE
+);
+
+SPIDER_THDVAR_VALUE_FUNC(bool, suppress_comment_ignored_warning)
+
+static MYSQL_THDVAR_BOOL(
+  ignore_comments,
+  PLUGIN_VAR_RQCMDARG,
+  "Whether to unconditionally ignore COMMENT and CONNECTION strings "
+  "without checking whether table options are specified",
+  NULL,
+  NULL,
+  FALSE
+);
+
+SPIDER_THDVAR_VALUE_FUNC(bool, ignore_comments)
+
 static struct st_mysql_storage_engine spider_storage_engine =
 { MYSQL_HANDLERTON_INTERFACE_VERSION };
 
@@ -2576,45 +2455,31 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(quick_page_byte),
   MYSQL_SYSVAR(low_mem_read),
   MYSQL_SYSVAR(select_column_mode),
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   MYSQL_SYSVAR(bgs_mode),
   MYSQL_SYSVAR(bgs_first_read),
   MYSQL_SYSVAR(bgs_second_read),
-#endif
   MYSQL_SYSVAR(first_read),
   MYSQL_SYSVAR(second_read),
   MYSQL_SYSVAR(crd_interval),
   MYSQL_SYSVAR(crd_mode),
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   MYSQL_SYSVAR(crd_sync),
-#endif
   MYSQL_SYSVAR(store_last_crd),
   MYSQL_SYSVAR(load_crd_at_startup),
   MYSQL_SYSVAR(crd_type),
   MYSQL_SYSVAR(crd_weight),
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   MYSQL_SYSVAR(crd_bg_mode),
-#endif
   MYSQL_SYSVAR(sts_interval),
   MYSQL_SYSVAR(sts_mode),
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   MYSQL_SYSVAR(sts_sync),
-#endif
   MYSQL_SYSVAR(store_last_sts),
   MYSQL_SYSVAR(load_sts_at_startup),
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   MYSQL_SYSVAR(sts_bg_mode),
-#endif
   MYSQL_SYSVAR(ping_interval_at_trx_start),
   MYSQL_SYSVAR(auto_increment_mode),
   MYSQL_SYSVAR(same_server_link),
   MYSQL_SYSVAR(local_lock_table),
   MYSQL_SYSVAR(use_pushdown_udf),
   MYSQL_SYSVAR(direct_dup_insert),
-  MYSQL_SYSVAR(udf_table_lock_mutex_count),
-  MYSQL_SYSVAR(udf_table_mon_mutex_count),
-  MYSQL_SYSVAR(udf_ds_bulk_insert_rows),
-  MYSQL_SYSVAR(udf_ds_table_loop_mode),
   MYSQL_SYSVAR(remote_access_charset),
   MYSQL_SYSVAR(remote_autocommit),
   MYSQL_SYSVAR(remote_time_zone),
@@ -2626,23 +2491,18 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(connect_mutex),
   MYSQL_SYSVAR(bka_engine),
   MYSQL_SYSVAR(bka_mode),
-  MYSQL_SYSVAR(udf_ct_bulk_insert_interval),
-  MYSQL_SYSVAR(udf_ct_bulk_insert_rows),
-  MYSQL_SYSVAR(use_handler),
   MYSQL_SYSVAR(error_read_mode),
   MYSQL_SYSVAR(error_write_mode),
   MYSQL_SYSVAR(skip_default_condition),
   MYSQL_SYSVAR(skip_parallel_search),
   MYSQL_SYSVAR(direct_order_limit),
   MYSQL_SYSVAR(read_only_mode),
-  MYSQL_SYSVAR(udf_ds_use_real_table),
   MYSQL_SYSVAR(general_log),
   MYSQL_SYSVAR(index_hint_pushdown),
   MYSQL_SYSVAR(max_connections),
   MYSQL_SYSVAR(conn_wait_timeout),
   MYSQL_SYSVAR(log_result_errors),
   MYSQL_SYSVAR(log_result_error_with_sql),
-  MYSQL_SYSVAR(version),
   MYSQL_SYSVAR(internal_xa_id_type),
   MYSQL_SYSVAR(casual_read),
   MYSQL_SYSVAR(dry_access),
@@ -2650,16 +2510,17 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(bka_table_name_type),
   MYSQL_SYSVAR(use_cond_other_than_pk_for_update),
   MYSQL_SYSVAR(connect_error_interval),
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   MYSQL_SYSVAR(table_sts_thread_count),
   MYSQL_SYSVAR(table_crd_thread_count),
-#endif
   MYSQL_SYSVAR(slave_trx_isolation),
   MYSQL_SYSVAR(remote_wait_timeout),
   MYSQL_SYSVAR(wait_timeout),
   MYSQL_SYSVAR(sync_sql_mode),
   MYSQL_SYSVAR(strict_group_by),
   MYSQL_SYSVAR(direct_aggregate),
+  MYSQL_SYSVAR(disable_group_by_handler),
+  MYSQL_SYSVAR(suppress_comment_ignored_warning),
+  MYSQL_SYSVAR(ignore_comments),
   NULL
 };
 
